/*

  Copyright (C) 2001 StrmnNrmn

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __DAEDALUSASSERT_H__
#define __DAEDALUSASSERT_H__

// Ideas for the ignored assert taken from Game Programming Gems I
#ifdef DAEDALUS_RELEASE_BUILD

#define DAEDALUS_ASSERT( e, msg )
#define DAEDALUS_ASSERT_Q( e )
#define DAEDALUS_ASSERTMSG( msg )

#else

int DaedalusAssert( const CHAR * msg, const CHAR * expression, const CHAR * file, UINT line );

//
// Use this to report on problems with the emulator - not the emulated title
//
#define DAEDALUS_ASSERT( e, msg )										\
{																		\
	static bool ignore = false;											\
	if ( !(e) && !ignore )												\
	{																	\
		int i = DaedalusAssert( (msg), #e,  __FILE__, __LINE__ );		\
		if ( i == 1 )													\
		{																\
			__asm { int 3 }	/* User breakpoint */						\
		}																\
		else if ( i == 0 )												\
		{																\
			ignore = true;	/* Ignore throughout session */				\
		}																\
	}																	\
}

//
// Use this to assert without specifying a message
//
#define DAEDALUS_ASSERT_Q( e )											\
{																		\
	static bool ignore = false;											\
	if ( !(e) && !ignore )												\
	{																	\
		int i = DaedalusAssert( "", #e, __FILE__, __LINE__ );			\
		if ( i == 1 )													\
		{																\
			__asm { int 3 }	/* User breakpoint */						\
		}																\
		else if ( i == 0 )												\
		{																\
			ignore = true;	/* Ignore throughout session */				\
		}																\
	}																	\
}

//
// Use this to assert unconditionally - e.g. for unhandled cases
//
#define DAEDALUS_ASSERTMSG( msg )										\
{																		\
	static bool ignore = false;											\
	if ( !ignore )														\
	{																	\
		int i = DaedalusAssert( msg, "", __FILE__, __LINE__ );			\
		if ( i == 1 )													\
		{																\
			__asm { int 3 }	/* User breakpoint */						\
		}																\
		else if ( i == 0 )												\
		{																\
			ignore = true;	/* Ignore throughout session */				\
		}																\
	}																	\
}



#endif

#endif // __DAEDALUSASSERT_H__
